//+------------------------------------------------------------------+
//|                                                            Otkat |
//|                                 Copyright  2009-2012, EarnForex |
//|                                        http://www.earnforex.com/ |
//+------------------------------------------------------------------+
#property copyright "Copyright  2009-2012, EarnForex"
#property link      "http://www.earnforex.com"
#property version   "1.03"

#property description "Otkat - expert advisor for trading on pull-backs."
#property description "ATTENTION: Will work properly only on H1 timeframe!"
#property description "ver. 1.02 - fixed ORDER_FILLING_CANCEL compile error."
#property description "ver. 1.03 - added ECN compatibility."

#include <Trade/Trade.mqh>

input int TakeProfit = 8;
input int StopLoss = 51;
input double Lots = 1.0;
input int Otkat = 20;
input int KoridorOC = 18;
input int KoridorOt = 3;
input int Slippage = 5;
//Set it to your MT5 server's timezone relative to GMT
//E.g. set it to 7 if your server is in GMT+7 timezone
input int Server_GMT_Difference = 1;
input bool ECN_Mode = false; // ECN_Mode: Set to true if stop-loss should be added only after Position Open

// Main trading object
CTrade *Trade;

int HourToCheck;
int HourToOpen = 0;

double Poin;
ulong Deviation;
int Multiplier;

void OnInit()
{
   //Checking for unconvetional Point digits number
   if ((_Point == 0.00001) || (_Point == 0.001))
   {
      Poin = _Point * 10;
      Deviation = Slippage * 10;
      Multiplier = 10;
   }
   else
   {
      Poin = _Point; //Normal
      Deviation = Slippage;
      Multiplier = 1;
   }

   if ((22 + Server_GMT_Difference) > 23)
   {
      HourToCheck = 22 + Server_GMT_Difference - 24;
   }
   else
   {
      HourToCheck = 22 + Server_GMT_Difference;
   }
   if (Server_GMT_Difference < 0) HourToOpen = 24 + Server_GMT_Difference;
   else HourToOpen = Server_GMT_Difference;
   
 	Trade = new CTrade;
	Trade.SetDeviationInPoints(Deviation);
}

//+------------------------------------------------------------------+
//| Expert Deinitialization Function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
{
	delete Trade;
}


//+------------------------------------------------------------------+
//| Checking time and date then posting order or closing position    |
//+------------------------------------------------------------------+
void OnTick()
{
   double OpCl, ClOp, ClLo, HiCl, OpD;
   MqlDateTime dt;

   if (ECN_Mode) DoSLTP();

   //Check number of bars
   if (Bars(_Symbol, _Period) < 100)
   {
      Print("Less than 100 bars.");
      return;  
   }
   
   //Check TP
   if (TakeProfit < SymbolInfoInteger(_Symbol, SYMBOL_TRADE_STOPS_LEVEL) / Multiplier)
   {
      Print("TakeProfit less than " + IntegerToString(SymbolInfoInteger(_Symbol, SYMBOL_TRADE_STOPS_LEVEL) / Multiplier));
      return;
   }

   //Check TP
   if (StopLoss < SymbolInfoInteger(_Symbol, SYMBOL_TRADE_STOPS_LEVEL) / Multiplier)
   {
      Print("StopLoss less than " + IntegerToString(SymbolInfoInteger(_Symbol, SYMBOL_TRADE_STOPS_LEVEL) / Multiplier));
      return;
   }

   //Closing position if necessary
   if (PositionsTotal() > 0)
   {
      if (PositionSelect(_Symbol))
      {
         TimeToStruct(TimeCurrent(), dt);
         if ((dt.hour == HourToCheck) && (dt.min > 45))
         {
            //If the open position is Long we have to send a Sell order to close it
            if (PositionGetInteger(POSITION_TYPE) == POSITION_TYPE_BUY)
            {
               fSell();
            }
            //If not then it's a Short position and we need to send Buy order
            else fBuy();
         }
      }
   }

   if (PositionsTotal() == 0) 
   {
     //Get the required timeseries
     double High[];
     double Low[];
     double Close[];
     double Open[];
     CopyHigh(_Symbol, _Period, 0, Bars(_Symbol, _Period), High);
     CopyLow(_Symbol, _Period, 0, Bars(_Symbol, _Period), Low);
     CopyClose(_Symbol, _Period, 0, Bars(_Symbol, _Period), Close);
     CopyOpen(_Symbol, _Period, 0, Bars(_Symbol, _Period), Open);
     ArraySetAsSeries(High, true);
     ArraySetAsSeries(Low, true);
     ArraySetAsSeries(Close, true);
     ArraySetAsSeries(Open, true);
     
     OpCl = Open[24] - Close[1];
     ClOp = Close[1] - Open[24];
     ClLo = Close[1] - Low[ArrayMinimum(Low, 0, 24)];
     HiCl = High[ArrayMaximum(High, 0, 24)] - Close[1];
     OpD = Open[0];
     
      //Check free margin
      if (AccountInfoDouble(ACCOUNT_FREEMARGIN) < (2000 * Lots))
      {
         Print("No money to trade. Free Margin = ", AccountInfoDouble(ACCOUNT_FREEMARGIN));
         return;
      }

      datetime tc = TimeCurrent();
      TimeToStruct(tc, dt);
      if (dt.year == 1970) return; //TimeCurrent errors filtering
      //Open positions only on midnight on H1 bar opening and NOT on Mondays/Sundays or Fridays/Saturdays
      if ((dt.hour == HourToOpen) && (dt.min <= 3) && (dt.day_of_week > 1) && (dt.day_of_week < 5))
      {
         //Two Long entry conditions
         if (((OpCl > KoridorOC * Poin) && (ClLo < (Otkat - KoridorOt) * Poin)) || ((ClOp > KoridorOC * Poin) && (HiCl > (Otkat + KoridorOt) * Poin)))
         {
            fBuy();
            return;
         }
         //Two Short entry conditions
         if (((ClOp > KoridorOC * Poin) && (HiCl < (Otkat - KoridorOt) * Poin)) || ((OpCl > KoridorOC * Poin) && (ClLo > (Otkat + KoridorOt) * Poin)))
         {
            fSell();
            return; 
         }
      }
   }
}

//+------------------------------------------------------------------+
//| Sell                                                             |
//+------------------------------------------------------------------+
void fSell()
{
	MqlTradeRequest Request;
	Request.action = TRADE_ACTION_DEAL;
	Request.symbol = _Symbol;
	Request.volume = Lots;
	Request.price = SymbolInfoDouble(Symbol(), SYMBOL_BID);
	Request.type = ORDER_TYPE_SELL; 
	if (!ECN_Mode)
	{
   	Request.sl = SymbolInfoDouble(Symbol(), SYMBOL_BID) + StopLoss * Poin;
   	Request.tp = SymbolInfoDouble(Symbol(), SYMBOL_BID) - TakeProfit * Poin;
   }
   else
   {
   	Request.sl = 0;
   	Request.tp = 0;
   }
   Request.type_filling = ORDER_FILLING_IOC;
   Request.deviation = Deviation;
   Request.comment = "Otkat";
   
   MqlTradeResult Result;
   
	bool result = OrderSend(Request, Result); 
	if (result == false)	Print("Incorrect Request/Result Structure when Buying.");
	else Print("Result Code: ", Result.retcode, ", ", "Deal: ", Result.deal, ", ", "Volume: ", Result.volume, ", ", "Price: ", Result.price, ", ", "Requote Bid: ", Result.bid, ", ", "Requote Ask: ", Result.ask);
}

//+------------------------------------------------------------------+
//| Close an order                                                   |
//+------------------------------------------------------------------+
void fBuy()
{
	MqlTradeRequest Request;
	Request.action = TRADE_ACTION_DEAL;
	Request.symbol = _Symbol;
	Request.volume = Lots;
	Request.price = SymbolInfoDouble(Symbol(), SYMBOL_ASK);
	Request.type = ORDER_TYPE_BUY; 
	if (!ECN_Mode)
	{
   	Request.sl = SymbolInfoDouble(Symbol(), SYMBOL_ASK) - StopLoss * Poin;
   	Request.tp = SymbolInfoDouble(Symbol(), SYMBOL_ASK) + TakeProfit * Poin;
   }
   else
   {
   	Request.sl = 0;
   	Request.tp = 0;
   }
   Request.type_filling = ORDER_FILLING_IOC;
   Request.deviation = Deviation;
   Request.comment = "Otkat";
   
   MqlTradeResult Result;
   
	bool result = OrderSend(Request, Result); 
	if (result == false)	Print("Incorrect Request/Result Structure when Selling.");
	else Print("Result Code: ", Result.retcode, ", ", "Deal: ", Result.deal, ", ", "Volume: ", Result.volume, ", ", "Price: ", Result.price, ", ", "Requote Bid: ", Result.bid, ", ", "Requote Ask: ", Result.ask);
}

//+------------------------------------------------------------------+
//| Applies SL and TP to open positions if ECN mode is on.           |
//+------------------------------------------------------------------+
void DoSLTP()
{
   if (!PositionSelect(_Symbol)) return;
   
   double SL = 0, TP = 0;
   
   if (PositionGetInteger(POSITION_TYPE) == POSITION_TYPE_BUY)
   {
      if (StopLoss > 0) SL = NormalizeDouble(PositionGetDouble(POSITION_PRICE_OPEN) - StopLoss * Poin, _Digits);
      if (TakeProfit > 0) TP = NormalizeDouble(PositionGetDouble(POSITION_PRICE_OPEN) + TakeProfit * Poin, _Digits);
   }
   if (PositionGetInteger(POSITION_TYPE) == POSITION_TYPE_SELL)
   {
      if (StopLoss > 0) SL = NormalizeDouble(PositionGetDouble(POSITION_PRICE_OPEN) + StopLoss * Poin, _Digits);
      if (TakeProfit > 0) TP = NormalizeDouble(PositionGetDouble(POSITION_PRICE_OPEN) - TakeProfit * Poin, _Digits);
   }

   if (((PositionGetDouble(POSITION_SL) != SL) || (PositionGetDouble(POSITION_TP) != TP)) && (PositionGetDouble(POSITION_SL) == 0) && (PositionGetDouble(POSITION_TP) == 0))
   {
      Trade.PositionModify(_Symbol, SL, TP);
   }
}
//+------------------------------------------------------------------+